#ifndef IND_STRATEGY_H
#define IND_STRATEGY_H

#include <eo> 
#include <ga.h>

#include "strategy.h" 



#include "strategies/KyptaStrategy.h"
#include "strategies/PilatStrategy.h"
#include "strategies/Radek_strategy.h"
#include "strategies/alcaponestg.h"
#include "strategies/dm_strategy.h"
#include "strategies/evelina_strategy.h"
#include "strategies/hip_strategy.h"
#include "strategies/jr_strategy.h"
#include "strategies/kovarik.h"
#include "strategies/langhammer.h"
#include "strategies/lessner_ulrych_3_strategy.h"
#include "strategies/ludek_cigler_strategy.h"
#include "strategies/moraj2am_strategy.h"
#include "strategies/my_strategy.h"
#include "strategies/ondra_k_str.h"
#include "strategies/pavel_ceska_strategy.h"
#include "strategies/vavrp4am_Strategy.h"


/*

Individual encapsulating strategy. 

*/ 

using namespace std;



class StrategyIndividual : public EO<double> {
public:
  StrategyIndividual() { myStrategy = NULL; }
  ~StrategyIndividual() { if (myStrategy) delete myStrategy; } 

  StrategyIndividual(const StrategyIndividual& right) {
    myStrategy = NULL;
    if (right.myStrategy)
      set(right.myStrategy->getName());
  }

  StrategyIndividual& operator=(const StrategyIndividual &right) {
    if (right.myStrategy)
      set(right.myStrategy->getName());
    else 
      myStrategy = NULL;
    return *this;
  } 

  Strategy& strategy() { return *myStrategy; } 

  void set(string name) {
    
    if (myStrategy) delete myStrategy; 
    
    if (name == "RandomStrategy") {  myStrategy = new RandomStrategy(); return; } 
    if (name == "AlwaysCooperate") {  myStrategy = new AlwaysCooperate(); return; } 
    if (name == "AlwaysDeceive") {  myStrategy = new AlwaysDeceive(); return; }
    if (name == "TFT_(CopyPartner)") {  myStrategy = new CopyPartner(); return; }
    if (name == "Alternate") {  myStrategy = new Alternate(); return; }
    if (name == "Pavlov") {  myStrategy = new Pavlov(); return; }
    if (name == "Spite") {  myStrategy = new Spite(); return; }
    if (name == "CCD") {  myStrategy = new CCD(); return; }
    if (name == "DDC") {  myStrategy = new DDC(); return; }
    if (name == "FQuick") {  myStrategy = new FQuick(); return; }
    if (name == "TF2T") {  myStrategy = new TF2T(); return; }
    
    if (name == "AlCapone")  { myStrategy =  new AlCaponeStrategy();  return; }
    if (name == "Hip")  { myStrategy =  new HipStrategy();  return; }
    if (name == "Revenger")  { myStrategy =  new Revenger();  return; }
    if (name == "Kypta")  { myStrategy =  new KyptaStrategy();  return; }
    if (name == "Langhammer")  { myStrategy =  new Langhammer();  return; }
    if (name == "JesteLepsiStrategie")  { myStrategy =  new JesteLepsiStrategie();  return; }
    if (name == "Ludek_Cigler")  { myStrategy =  new LudekCiglerStrategy();  return; }
    if (name == "Ondra1")  { myStrategy =  new Ondra1();  return; }
    if (name == "PilatStrategie")  { myStrategy =  new PilatStrategy();  return; }
    if (name == "vavrp4am_Strategy")  { myStrategy =  new vavrp4am_Strategy();  return; }
    if (name == "EvelinaStrategy")  { myStrategy =  new EvelinaStrategy();  return; }


    if (name == "DM")  { myStrategy =  new DM_Strategy();  return; }
    if (name == "JrStrategy")  { myStrategy =  new JrStrategy();  return; }
    if (name == "moraj2amSTRG")  { myStrategy =  new MORAJ2AMSTRG();  return; }
    if (name == "Pavel_Ceska_Strategy")  { myStrategy =  new MyStrategy_pavel_ceska();  return; }
    if (name == "RadekStrategy")  { myStrategy =  new RadekStrategy();  return; }

    
     throw std::runtime_error("unknown strategy: "+name);
    
  }

 

  virtual void printOn(std::ostream &os) const {
    os << myStrategy->getName(); 
  }


  Strategy* myStrategy; // pointer to strategy
};  


typedef StrategyIndividual Indi; 


#endif
