#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


// standard includes
#include <stdexcept>  // runtime_error 
#include <iostream>   // cout
#ifdef HAVE_SSTREAM
#include <sstream>
#else
#include <strstream>  // ostrstream, istrstream
#endif

// the general include for eo

#include <eo>
#include <ga.h>


#include "fitness.h"
#include "gui/gui.h"
#include "breed.h"





using namespace std;


void main_function(int argc, char **argv)
{

  vector<string> stratNames;
  stratNames.push_back("RandomStrategy");
  stratNames.push_back("AlwaysCooperate");
  stratNames.push_back("AlwaysDeceive");
  stratNames.push_back("TFT_(CopyPartner)");
  stratNames.push_back("Alternate");
  stratNames.push_back("Pavlov");
  stratNames.push_back("Spite");
  stratNames.push_back("CCD");
  stratNames.push_back("DDC");
  stratNames.push_back("FQuick");
  stratNames.push_back("TF2T");
  
  // strategie ze cviceni 
  stratNames.push_back("Kypta");
  stratNames.push_back("PilatStrategie");
  stratNames.push_back("RadekStrategy");
  stratNames.push_back("AlCapone");
  stratNames.push_back("DM");
  stratNames.push_back("EvelinaStrategy");
  stratNames.push_back("Hip");
  stratNames.push_back("JrStrategy");
  stratNames.push_back("Revenger");
  stratNames.push_back("Langhammer");
  stratNames.push_back("JesteLepsiStrategie");
  stratNames.push_back("Ludek_Cigler");
  stratNames.push_back("moraj2amSTRG");
  stratNames.push_back("Ondra1");
  stratNames.push_back("Pavel_Ceska_Strategy");
  stratNames.push_back("vavrp4am_Strategy");


  vector<int> number; 
  number.resize( stratNames.size() );
  for (unsigned i=0; i<stratNames.size(); i++) 
    number[i] = 1; 


   unsigned int SEED;                   // seed for random number generator
   unsigned int MAX_GEN;                // Maximum number of generation before STOP


   // PARAMETRES

    eoParser parser(argc, argv);
    
    eoValueParam<uint32_t> seedParam(time(0), "seed", "Random number seed", 'S');
    parser.processParam( seedParam );
    SEED = seedParam.value();
    
    eoValueParam<unsigned int> maxGenParam(500, "maxGen", "Maximum number of generations",'G');
    parser.processParam( maxGenParam, "Stopping criterion" );
    MAX_GEN = maxGenParam.value();


    string str_status = parser.ProgramName() + ".status"; 
    eoValueParam<string> statusParam(str_status.c_str(), "status","Status file",'S');
    parser.processParam( statusParam, "Persistence" );

    
    if (parser.userNeedsHelp())
      {
        parser.printHelp(cout);
        exit(1);
      }
    if (statusParam.value() != "")
      {
	ofstream os(statusParam.value().c_str());
	os << parser;
      }


    //////////////////////////
    //  Random seed
    //////////////////////////
    rng.reseed(SEED);

    /////////////////////////////
    // Fitness function
    ////////////////////////////
    DummyFitness eval;


    myGui gui(50,330); // dolni a horni mez pro graf fitness 
    
    gui.do_select(stratNames,number); 

    gui.setup(stratNames,number);
 
    ////////////////////////////////
    // Initilisation of population
    ////////////////////////////////
    eoPop<Indi> pop;
    for (unsigned i=0; i<stratNames.size(); i++) 
      for (int j=0; j<number[i]; j++) {
	Indi v; 
	v.set(stratNames[i]);
	//	v.fitness( 0.0 );
	pop.push_back(v);
      }

    cout << "Created " << endl;
    eval(pop,pop); 

    // OUTPUT
    //    pop.sort();
    cout << "Initial Population" << endl;
    cout << pop;

    /////////////////////////////////////
    // breed 
    ////////////////////////////////////
    MyBreed breed(stratNames); 
    
    eoGenerationalReplacement<Indi> replace; 
 

    //////////////////////////////////////
    // termination condition
    /////////////////////////////////////
        
    // eoGenContinue<Indi> genCont(100);
    eoCheckPoint<Indi> checkpoint(gui);    

     eoValueParam<unsigned> generationCounter(0, "Gen.");
     eoIncrementor<unsigned> increment(generationCounter.value());
     checkpoint.add(increment);

       
     
     /////////////////////////////////////////
     // the algorithm
     ////////////////////////////////////////
     eoEasyEA<Indi> gga(checkpoint, 
			eval, 
			breed, 
			replace);


     // Apply algo to pop - that's it!
     gga(pop);
  
     // OUTPUT
     //     pop.sort();
     cout << "FINAL Population\n" << pop << endl;
     cout << pop; 
}
 

int main(int argc, char **argv)
{

    try
    {
        main_function(argc, argv);
    }
    catch(exception& e)
    {
        cout << "Exception: " << e.what() << '\n';
    }

    return 1;
}
